{*************************************************************************
 *                                                                       *
 * Programm:  FIR_Main (Hauptprogramm)                                   *
 *                                                                       *
 * Quelltext: FIR_Main.DSP                                               *
 *                                                                       *
 * Funktion:  Transversales FIR-Filter nach der Gleichung                *
 *                                                                       *
 *               y[n] = b0 * x[n] + b1 * x[n-1] + b2 * x[n-2]            *
 *                                                                       *
 * Parameter: Filterkoeffizienten b0, b1 b2                              *
 *            Die Werte werden beim Linken aus Daten-File "koef.dat"     *
 *            initilisiert (16-bit Hex-wert linksbuendig in 24-bit       *
 *            Hex-Wert getrennt durch CRTN).                             *
 *                                                                       *
 * Daten-                                                                *
 * Eingang:   16 Werte x[n] im Datenspeicherfeld data[].                 *
 *            Das Feld wird beim Linken des Programms initialisiert, mit *
 *            den Werten aus der Daten-Datei "daten.dat" (16-bit Hex     *
 *            linksbuendig in 24-bit Hex-Wert getrennt durch CRTN).      *
 *                                                                       *
 * Daten-                                                                *
 * Ausgang:   14 berechnete Werte y[n] im Datenspeicherfeld erg[],       *
 *            wobei die Elemente y[0],y[1] 0 sind und der erste          *
 *            berechnete Wert auf Speicherstelle y[2] steht.             *
 *                                                                       *
 *
 *************************************************************************}


                                    { A  Definitionen                     }
                                    { ---------------------------------A--}
  
.MODULE/RAM/ABS=0       FIR_Main;             { Modulname und absolute Addresse}
.EXTERNAL               fir;                  { Filterfunktion in extra Modul  }
.EXTERNAL               adap_fkt;                  { Filterfunktion in extra Modul  }

.CONST                  WERTE=16;             { Anzahl der zugelinkten Werte   }
.CONST                  N=3;                  { Anzahl der Filterkoef.         }
.CONST                  STEP =1;

.VAR/DM/RAM/CIRC        data[WERTE];          { Feld fuer zu bearbeitende Daten}
.VAR/DM/RAM/CIRC        d[WERTE];             { Desire Vector                  }
.VAR/DM/RAM/CIRC        erg[WERTE];           { Feld fuer bearbeitete Daten    }
.VAR/DM/RAM/CIRC        e[WERTE];

.VAR/DM/CIRC            delayBuffer[N];       { VerzoegerungsPuffer            }
                                    { B Initialisierungen beim Linken     }
.VAR/PM/CIRC            coef_buffer[96];              { Filter-Koeff.-Feld anlegen     }
                                    { ---------------------------------B--}

.INIT                   coef_buffer:<coefbuf.DAT>;          { koef.dat wird in coef[] gelinkt}
.INIT                   data:<daten.dat>;         { daten.dat wird in data[] gel.  }
.INIT                   d:<desire.dat>;             { desire.dat wird in d[k] gel.   }
.INIT                   erg :<null.dat>;          { erg[] wird mit Nullen initial. }
.INIT                   e :<null.dat>;          { erg[] wird mit Nullen initial. }


                                    { C Interrupt Anweisungen             }
                                    { ---------------------------------C--}

 JUMP start; RTI; RTI; RTI;              { falls Reset gehe zu Lbl.'start'}
 RTI; RTI; RTI; RTI;                     { falls Interruptrequest 2 return}
 RTI; RTI; RTI; RTI;                     { ...   SPORT0 TX                }
 RTI; RTI; RTI; RTI;                     { ...   SPORT0 RX                }
 RTI; RTI; RTI; RTI;                     { ...   IRQ1                     }
 RTI; RTI; RTI; RTI;                     { ...   IRQ0                     }
 RTI; RTI; RTI; RTI;                     { ...   TIMER                    }
 
 
 
				    { D Verzoegerungspuffer nullsetzen    }
				    { ---------------------------------D--}

start:     I3 =  ^delayBuffer;          { -> I3 Adresse delayBuffer       }
	   M3 = 1;
	   M5 = 0;
	   M6 = 1;
	   L3 =  %delayBuffer;          { -> L3 Laenge  delayBuffer       }
	   
	   AX0 = 0;
	   CNTR = N;
	   DO iniBuffer UNTIL CE;       { i = 0 -> (N-1)                  }
iniBuffer: DM(I3,M3)=AX0;               { Null in den Verzoegerungspuffer }
	   
				    { E Filterschleife vorbereiten        }
				    { ---------------------------------E--}




	   I1 = ^data; 
           L1=%data;                    { -> Eingangsvektor x[0]          }

	   I2 = ^erg;
           L2=%erg;                     { -> Ausgangsvektor y[0]          }

	   I4 = ^coef_buffer;           { -> FIR-Koeffizientenvektor      }
	   L4 = %coef_buffer;           {    zirkulaere  Adressierung     }
                                        {    zum lesen der coef.      }

	   I3 = ^delayBuffer;           { -> Verzoegerungspuffer          }
	   L3 = %delayBuffer;           {    zirkulaere  Adressierung     }

	   I5 = ^d;                     { -> desire_vector                 }
	   L5 = %d;                     {    zirkulaere  Adressierung     }

           I6 = ^e;                     { -> e[k]                         }
	   L6 = %e;                     {      zirkulaere  Adressierung   }

	   I7 = ^coef_buffer;           { -> FIR-Koeffizientenvektor      }
	   L7 = %coef_buffer;           {    zirkulaere  Adressierung     }
                                        {    zum schreiben der coef.      }

           MODIFY(I7,M7);               { I7 in die Mitte des SP-B. setzen}


	   M0 = 0;                      { Modifier Mx definieren          }
	   M1 = 1;
	   M2 = -1;
	   M5 = 1;
	   M6 = 0;
           
           ENA M_MODE;                  { Integer Mode einschalten        }
				    { F Filterschleife ausfuehren         }
				    { ---------------------------------F--}
           CNTR = 50;
           do fil UNTIL CE;             {Filterzyklen}
           CNTR = WERTE;                { Anzahl der Eingangswerte        }

	   DO filter UNTIL CE;           {Filtertakte}
	      MX0=DM(I1,M0);              { MX0 ist Eingangswert des Filters }
	      MY0 = N-1;                  { MY0 ist intern. Schleifenzaehler }
	      AX0 = N-1;
	      CALL fir;                   { Filteraufruf pro Eingangswert    }
              DM(I2,M1)=MR0;              { MR0 ist Ausgangswert des Filters }
              M7 = -3;
              MODIFY(I4,M7);
              AX0  = dm(I5,M5);           {AXO = d[k]                        }
              AY0 = MR0;                  {AY0 = y[k]                        }
              AR = AX0 - AY0;             {e[k] = d[k] - y[k]                }
              DM(I6,M5) = AR;

              MX0=DM(I1,M3);              { MX0 ist Eingangswert des Filters }
              MX1 = AR;                   {Parameter MX1 = e[k]              }
              MY1 = STEP;                 {Parameter MY1 = STEP              }
              MY0 = N-1;                  { MY0 ist intern. Schleifenzaehler }
       filter:CALL adap_fkt;              {adap coeff                        }
    fil:   nop;

	   idle;                       { (Ende) Warten auf Interrupt      }

.ENDMOD;


